# Generated by ChatGPT on 2025-10-06

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("indy_hub", "0018_enhanced_production_simulation_models"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="charactersettings",
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name="charactersettings",
            constraint=models.UniqueConstraint(
                fields=("user", "character_id"),
                name="charsettings_user_char_uq",
            ),
        ),
        migrations.AddIndex(
            model_name="charactersettings",
            index=models.Index(
                fields=("user", "character_id"),
                name="charsettings_user_char_idx",
            ),
        ),
    ]
